@@ -14,6 +14,7 @@ using Avalonia;
 using Avalonia.Controls;
 using Avalonia.Controls.ApplicationLifetimes;
 using Avalonia.Data.Core.Plugins;
+using Avalonia.Input.Platform;
 using Avalonia.Markup.Xaml;
 using Avalonia.Media;
 using Avalonia.Media.Fonts;
@@ -105,7 +106,7 @@ namespace SourceGit
         #endregion
 
         #region Utility Functions
-        public static object CreateViewForViewModel(object data)
+        public static Control CreateViewForViewModel(object data)
         {
             var dataTypeName = data.GetType().FullName;
             if (string.IsNullOrEmpty(dataTypeName) || !dataTypeName.Contains(".ViewModels.", StringComparison.Ordinal))
@@ -114,7 +115,7 @@ namespace SourceGit
             var viewTypeName = dataTypeName.Replace(".ViewModels.", ".Views.");
             var viewType = Type.GetType(viewTypeName);
             if (viewType != null)
-                return Activator.CreateInstance(viewType);
+                return Activator.CreateInstance(viewType) as Control;
 
             return null;
         }
@@ -261,7 +262,7 @@ namespace SourceGit
             }
         }
 
-        public static void SetFonts(string defaultFont, string monospaceFont, bool onlyUseMonospaceFontInEditor)
+        public static void SetFonts(string defaultFont, string monospaceFont)
         {
             if (Current is not App app)
                 return;
@@ -284,7 +285,7 @@ namespace SourceGit
                 if (!string.IsNullOrEmpty(defaultFont))
                 {
                     monospaceFont = $"fonts:SourceGit#JetBrains Mono,{defaultFont}";
-                    resDic.Add("Fonts.Monospace", new FontFamily(monospaceFont));
+                    resDic.Add("Fonts.Monospace", FontFamily.Parse(monospaceFont));
                 }
             }
             else
@@ -292,20 +293,7 @@ namespace SourceGit
                 if (!string.IsNullOrEmpty(defaultFont) && !monospaceFont.Contains(defaultFont, StringComparison.Ordinal))
                     monospaceFont = $"{monospaceFont},{defaultFont}";
 
-                resDic.Add("Fonts.Monospace", new FontFamily(monospaceFont));
-            }
-
-            if (onlyUseMonospaceFontInEditor)
-            {
-                if (string.IsNullOrEmpty(defaultFont))
-                    resDic.Add("Fonts.Primary", new FontFamily("fonts:Inter#Inter"));
-                else
-                    resDic.Add("Fonts.Primary", new FontFamily(defaultFont));
-            }
-            else
-            {
-                if (!string.IsNullOrEmpty(monospaceFont))
-                    resDic.Add("Fonts.Primary", new FontFamily(monospaceFont));
+                resDic.Add("Fonts.Monospace", FontFamily.Parse(monospaceFont));
             }
 
             if (resDic.Count > 0)
@@ -324,7 +312,7 @@ namespace SourceGit
         public static async Task<string> GetClipboardTextAsync()
         {
             if (Current?.ApplicationLifetime is IClassicDesktopStyleApplicationLifetime { MainWindow.Clipboard: { } clipboard })
-                return await clipboard.GetTextAsync();
+                return await clipboard.TryGetTextAsync();
             return null;
         }
 
@@ -382,7 +370,7 @@ namespace SourceGit
 
             SetLocale(pref.Locale);
             SetTheme(pref.Theme, pref.ThemeOverrides);
-            SetFonts(pref.DefaultFontFamily, pref.MonospaceFontFamily, pref.OnlyUseMonoFontInEditor);
+            SetFonts(pref.DefaultFontFamily, pref.MonospaceFontFamily);
         }
 
         public override void OnFrameworkInitializationCompleted()
@@ -399,6 +387,9 @@ namespace SourceGit
                         e.Cancel = true;
                 });
 
+                if (TryLaunchAsFileHistoryViewer(desktop))
+                    return;
+
                 if (TryLaunchAsCoreEditor(desktop))
                     return;
 
@@ -525,6 +516,33 @@ namespace SourceGit
             return true;
         }
 
+        private bool TryLaunchAsFileHistoryViewer(IClassicDesktopStyleApplicationLifetime desktop)
+        {
+            var args = desktop.Args;
+            if (args is not { Length: > 1 } || !args[0].Equals("--file-history", StringComparison.Ordinal))
+                return false;
+
+            var file = Path.GetFullPath(args[1]);
+            var dir = Path.GetDirectoryName(file);
+
+            var test = new Commands.QueryRepositoryRootPath(dir).GetResult();
+            if (!test.IsSuccess || string.IsNullOrEmpty(test.StdOut))
+            {
+                Console.Out.WriteLine($"'{args[1]}' is not in a valid git repository");
+                desktop.Shutdown(-1);
+                return true;
+            }
+
+            var repo = test.StdOut.Trim();
+            var relFile = Path.GetRelativePath(repo, file);
+            var viewer = new Views.FileHistories()
+            {
+                DataContext = new ViewModels.FileHistories(repo, relFile)
+            };
+            desktop.MainWindow = viewer;
+            return true;
+        }
+
         private bool TryLaunchAsCoreEditor(IClassicDesktopStyleApplicationLifetime desktop)
         {
             var args = desktop.Args;
@@ -675,9 +693,8 @@ namespace SourceGit
                 if (string.IsNullOrEmpty(t))
                     continue;
 
-                // Collapse multiple spaces into single space
-                var prevChar = '\0';
                 var sb = new StringBuilder();
+                var prevChar = '\0';
 
                 foreach (var c in t)
                 {
@@ -688,14 +705,16 @@ namespace SourceGit
                 }
 
                 var name = sb.ToString();
-                if (name.Contains('#'))
+                try
                 {
-                    if (!name.Equals("fonts:Inter#Inter", StringComparison.Ordinal) &&
-                        !name.Equals("fonts:SourceGit#JetBrains Mono", StringComparison.Ordinal))
-                        continue;
+                    var fontFamily = FontFamily.Parse(name);
+                    if (fontFamily.FamilyTypefaces.Count > 0)
+                        trimmed.Add(name);
+                }
+                catch
+                {
+                    // Ignore exceptions.
                 }
-
-                trimmed.Add(name);
             }
 
             return trimmed.Count > 0 ? string.Join(',', trimmed) : string.Empty;
