@@ -1,12 +1,15 @@
-﻿namespace SourceGit.Commands
+﻿using System.Threading.Tasks;
+
+namespace SourceGit.Commands
 {
     public class Pull : Command
     {
         public Pull(string repo, string remote, string branch, bool useRebase)
         {
+            _remote = remote;
+
             WorkingDirectory = repo;
             Context = repo;
-            SSHKey = new Config(repo).Get($"remote.{remote}.sshkey");
             Args = "pull --verbose --progress ";
 
             if (useRebase)
@@ -14,5 +17,13 @@
 
             Args += $"{remote} {branch}";
         }
+
+        public async Task<bool> RunAsync()
+        {
+            SSHKey = await new Config(WorkingDirectory).GetAsync($"remote.{_remote}.sshkey").ConfigureAwait(false);
+            return await ExecAsync().ConfigureAwait(false);
+        }
+
+        private readonly string _remote;
     }
 }
