@@ -11,9 +11,19 @@ namespace SourceGit.Commands
             Args = $"rev-parse {refname}";
         }
 
+        public string GetResult()
+        {
+            return Parse(ReadToEnd());
+        }
+
         public async Task<string> GetResultAsync()
         {
             var rs = await ReadToEndAsync().ConfigureAwait(false);
+            return Parse(rs);
+        }
+
+        private string Parse(Result rs)
+        {
             if (rs.IsSuccess && !string.IsNullOrEmpty(rs.StdOut))
                 return rs.StdOut.Trim();
 
