@@ -58,7 +58,7 @@ namespace SourceGit.ViewModels
 
             if (!DiscardLocalChanges)
             {
-                var changes = await new Commands.CountLocalChangesWithoutUntracked(_repo.FullPath).GetResultAsync();
+                var changes = await new Commands.CountLocalChanges(_repo.FullPath, false).GetResultAsync();
                 if (changes > 0)
                 {
                     succ = await new Commands.Stash(_repo.FullPath)
