@@ -109,9 +109,23 @@ namespace SourceGit.ViewModels
                         pixelFormat = PixelFormats.Gray8;
                         break;
                     case ImageFormat.R5g5b5:
-                    case ImageFormat.R5g5b5a1:
                         pixelFormat = PixelFormats.Bgr555;
                         break;
+                    case ImageFormat.R5g5b5a1:
+                        var pixels1 = pfiImage.DataLen / 2;
+                        data = new byte[pixels1 * 4];
+                        stride = pfiImage.Width * 4;
+                        for (var i = 0; i < pixels1; i++)
+                        {
+                            var src = BitConverter.ToUInt16(pfiImage.Data, i * 2);
+                            data[i * 4 + 0] = (byte)Math.Round((src & 0x1F) / 31F * 255); // B
+                            data[i * 4 + 1] = (byte)Math.Round(((src >> 5) & 0x1F) / 31F * 255); // G
+                            data[i * 4 + 2] = (byte)Math.Round(((src >> 10) & 0x1F) / 31F * 255); // R
+                            data[i * 4 + 3] = (byte)((src >> 15) * 255F); // A
+                        }
+
+                        alphaFormat = AlphaFormat.Unpremul;
+                        break;
                     case ImageFormat.R5g6b5:
                         pixelFormat = PixelFormats.Bgr565;
                         break;
@@ -131,10 +145,10 @@ namespace SourceGit.ViewModels
                             data[i * 4 + 3] = (byte)Math.Round(((src >> 12) & 0x0F) / 15F * 255); // A
                         }
 
-                        alphaFormat = AlphaFormat.Premul;
+                        alphaFormat = AlphaFormat.Unpremul;
                         break;
                     case ImageFormat.Rgba32:
-                        alphaFormat = AlphaFormat.Premul;
+                        alphaFormat = AlphaFormat.Unpremul;
                         break;
                     default:
                         return new ImageSource(null, 0);
@@ -155,17 +169,18 @@ namespace SourceGit.ViewModels
                 if (tiff == null)
                     return new ImageSource(null, 0);
 
+                // Currently only supports image when its `BITSPERSAMPLE` is one in [1,2,4,8,16]
                 var width = tiff.GetField(TiffTag.IMAGEWIDTH)[0].ToInt();
                 var height = tiff.GetField(TiffTag.IMAGELENGTH)[0].ToInt();
                 var pixels = new int[width * height];
-
-                // Currently only supports image when its `BITSPERSAMPLE` is one in [1,2,4,8,16]
                 tiff.ReadRGBAImageOriented(width, height, pixels, Orientation.TOPLEFT);
 
-                var ptr = Marshal.UnsafeAddrOfPinnedArrayElement(pixels, 0);
                 var pixelSize = new PixelSize(width, height);
                 var dpi = new Vector(96, 96);
-                var bitmap = new Bitmap(PixelFormats.Rgba8888, AlphaFormat.Premul, ptr, pixelSize, dpi, width * 4);
+                var bitmap = new WriteableBitmap(pixelSize, dpi, PixelFormats.Rgba8888, AlphaFormat.Unpremul);
+
+                using var frameBuffer = bitmap.Lock();
+                Marshal.Copy(pixels, 0, frameBuffer.Address, pixels.Length);
                 return new ImageSource(bitmap, size);
             }
         }
