@@ -35,6 +35,7 @@ namespace SourceGit.ViewModels
             {
                 if (SetProperty(ref _activePage, value))
                 {
+                    CancelCommandPalette();
                     UpdateTitle();
 
                     if (!_ignoreIndexChange && value is { Data: Repository repo })
@@ -43,10 +44,10 @@ namespace SourceGit.ViewModels
             }
         }
 
-        public IDisposable Switcher
+        public ICommandPalette CommandPalette
         {
-            get => _switcher;
-            private set => SetProperty(ref _switcher, value);
+            get => _commandPalette;
+            set => SetProperty(ref _commandPalette, value);
         }
 
         public Launcher(string startupRepo)
@@ -127,22 +128,6 @@ namespace SourceGit.ViewModels
             _ignoreIndexChange = false;
         }
 
-        public void OpenWorkspaceSwitcher()
-        {
-            Switcher = new WorkspaceSwitcher(this);
-        }
-
-        public void OpenTabSwitcher()
-        {
-            Switcher = new LauncherPageSwitcher(this);
-        }
-
-        public void CancelSwitcher()
-        {
-            Switcher?.Dispose();
-            Switcher = null;
-        }
-
         public void SwitchWorkspace(Workspace to)
         {
             if (to == null || to.IsActive)
@@ -269,6 +254,7 @@ namespace SourceGit.ViewModels
                     last.Popup = null;
                     UpdateTitle();
 
+                    CancelCommandPalette();
                     GC.Collect();
                 }
                 else
@@ -392,6 +378,20 @@ namespace SourceGit.ViewModels
                 ActiveWorkspace.ActiveIdx = ActiveWorkspace.Repositories.IndexOf(node.Id);
         }
 
+        public void OpenCommandPalette(ICommandPalette commandPalette)
+        {
+            var old = _commandPalette;
+            CommandPalette = commandPalette;
+            old?.Dispose();
+        }
+
+        public void CancelCommandPalette()
+        {
+            _commandPalette?.Dispose();
+            CommandPalette = null;
+            GC.Collect();
+        }
+
         public void DispatchNotification(string pageId, string message, bool isError)
         {
             if (!Dispatcher.UIThread.CheckAccess())
@@ -497,6 +497,6 @@ namespace SourceGit.ViewModels
         private LauncherPage _activePage = null;
         private bool _ignoreIndexChange = false;
         private string _title = string.Empty;
-        private IDisposable _switcher = null;
+        private ICommandPalette _commandPalette = null;
     }
 }
