@@ -41,10 +41,15 @@ namespace SourceGit.ViewModels
             set;
         } = new();
 
-        public Models.FilterMode HistoriesFilterMode
+        public Models.HistoryFilterCollection HistoryFilterCollection
         {
-            get => _historiesFilterMode;
-            private set => SetProperty(ref _historiesFilterMode, value);
+            get => _historyFilterCollection;
+        }
+
+        public Models.FilterMode HistoryFilterMode
+        {
+            get => _historyFilterMode;
+            private set => SetProperty(ref _historyFilterMode, value);
         }
 
         public bool HasAllowedSignersFile
@@ -258,91 +263,24 @@ namespace SourceGit.ViewModels
             }
         }
 
-        public bool IsSearching
+        public bool IsSearchingCommits
         {
-            get => _isSearching;
+            get => _isSearchingCommits;
             set
             {
-                if (SetProperty(ref _isSearching, value))
+                if (SetProperty(ref _isSearchingCommits, value))
                 {
                     if (value)
-                    {
                         SelectedViewIndex = 0;
-                        CalcWorktreeFilesForSearching();
-                    }
                     else
-                    {
-                        SearchedCommits = new List<Models.Commit>();
-                        SelectedSearchedCommit = null;
-                        SearchCommitFilter = string.Empty;
-                        MatchedFilesForSearching = null;
-                        _requestingWorktreeFiles = false;
-                        _worktreeFiles = null;
-                    }
+                        _searchCommitContext.EndSearch();
                 }
             }
         }
 
-        public bool IsSearchLoadingVisible
+        public SearchCommitContext SearchCommitContext
         {
-            get => _isSearchLoadingVisible;
-            private set => SetProperty(ref _isSearchLoadingVisible, value);
-        }
-
-        public bool OnlySearchCommitsInCurrentBranch
-        {
-            get => _onlySearchCommitsInCurrentBranch;
-            set
-            {
-                if (SetProperty(ref _onlySearchCommitsInCurrentBranch, value) && !string.IsNullOrEmpty(_searchCommitFilter))
-                    StartSearchCommits();
-            }
-        }
-
-        public int SearchCommitFilterType
-        {
-            get => _searchCommitFilterType;
-            set
-            {
-                if (SetProperty(ref _searchCommitFilterType, value))
-                {
-                    CalcWorktreeFilesForSearching();
-                    if (!string.IsNullOrEmpty(_searchCommitFilter))
-                        StartSearchCommits();
-                }
-            }
-        }
-
-        public string SearchCommitFilter
-        {
-            get => _searchCommitFilter;
-            set
-            {
-                if (SetProperty(ref _searchCommitFilter, value) && IsSearchingCommitsByFilePath())
-                    CalcMatchedFilesForSearching();
-            }
-        }
-
-        public List<string> MatchedFilesForSearching
-        {
-            get => _matchedFilesForSearching;
-            private set => SetProperty(ref _matchedFilesForSearching, value);
-        }
-
-        public List<Models.Commit> SearchedCommits
-        {
-            get => _searchedCommits;
-            set => SetProperty(ref _searchedCommits, value);
-        }
-
-        public Models.Commit SelectedSearchedCommit
-        {
-            get => _selectedSearchedCommit;
-            set
-            {
-                if (SetProperty(ref _selectedSearchedCommit, value) && value != null)
-                    NavigateToCommit(value.SHA);
-            }
+            get => _searchCommitContext;
         }
 
         public bool IsLocalBranchGroupExpanded
@@ -526,6 +464,24 @@ namespace SourceGit.ViewModels
                 _settings = new Models.RepositorySettings();
             }
 
+            var historyFilterFile = Path.Combine(GitDir, "sourcegit.filters");
+            if (File.Exists(historyFilterFile))
+            {
+                try
+                {
+                    using var stream = File.OpenRead(historyFilterFile);
+                    _historyFilterCollection = JsonSerializer.Deserialize(stream, JsonCodeGen.Default.HistoryFilterCollection);
+                }
+                catch
+                {
+                    _historyFilterCollection = new Models.HistoryFilterCollection();
+                }
+            }
+            else
+            {
+                _historyFilterCollection = new Models.HistoryFilterCollection();
+            }
+
             try
             {
                 _watcher = new Models.Watcher(this, FullPath, _gitCommonDir);
@@ -535,14 +491,11 @@ namespace SourceGit.ViewModels
                 App.RaiseException(string.Empty, $"Failed to start watcher for repository: '{FullPath}'. You may need to press 'F5' to refresh repository manually!\n\nReason: {ex.Message}");
             }
 
-            if (_settings.HistoriesFilters.Count > 0)
-                _historiesFilterMode = _settings.HistoriesFilters[0].Mode;
-            else
-                _historiesFilterMode = Models.FilterMode.None;
-
+            _historyFilterMode = _historyFilterCollection.Mode;
             _histories = new Histories(this);
             _workingCopy = new WorkingCopy(this) { CommitMessage = _settings.LastCommitMessage };
             _stashesPage = new StashesPage(this);
+            _searchCommitContext = new SearchCommitContext(this);
 
             if (Preferences.Instance.ShowLocalChangesByDefault)
             {
@@ -565,11 +518,35 @@ namespace SourceGit.ViewModels
             SelectedView = null; // Do NOT modify. Used to remove exists widgets for GC.Collect
             Logs.Clear();
 
-            if (!_isWorktree)
+            if (!_isWorktree && Directory.Exists(_gitCommonDir))
             {
-                _settings.LastCommitMessage = _workingCopy.CommitMessage;
-                using var stream = File.Create(Path.Combine(_gitCommonDir, "sourcegit.settings"));
-                JsonSerializer.Serialize(stream, _settings, JsonCodeGen.Default.RepositorySettings);
+                try
+                {
+                    if (_workingCopy.InProgressContext != null && !string.IsNullOrEmpty(_workingCopy.CommitMessage))
+                        File.WriteAllText(Path.Combine(GitDir, "MERGE_MSG"), _workingCopy.CommitMessage);
+                    else
+                        _settings.LastCommitMessage = _workingCopy.CommitMessage;
+
+                    using var stream = File.Create(Path.Combine(_gitCommonDir, "sourcegit.settings"));
+                    JsonSerializer.Serialize(stream, _settings, JsonCodeGen.Default.RepositorySettings);
+                }
+                catch (Exception)
+                {
+                    // Ignore
+                }
+            }
+
+            if (Directory.Exists(GitDir))
+            {
+                try
+                {
+                    using var stream = File.Create(Path.Combine(GitDir, "sourcegit.filters"));
+                    JsonSerializer.Serialize(stream, _historyFilterCollection, JsonCodeGen.Default.HistoryFilterCollection);
+                }
+                catch
+                {
+                    // Ignore
+                }
             }
 
             if (_cancellationRefreshBranches is { IsCancellationRequested: false })
@@ -587,12 +564,14 @@ namespace SourceGit.ViewModels
             _autoFetchTimer = null;
 
             _settings = null;
-            _historiesFilterMode = Models.FilterMode.None;
+            _historyFilterCollection = null;
+            _historyFilterMode = Models.FilterMode.None;
 
             _watcher?.Dispose();
             _histories.Dispose();
             _workingCopy.Dispose();
             _stashesPage.Dispose();
+            _searchCommitContext.Dispose();
 
             _watcher = null;
             _histories = null;
@@ -610,12 +589,6 @@ namespace SourceGit.ViewModels
             _visibleTags = null;
             _submodules.Clear();
             _visibleSubmodules = null;
-            _searchedCommits.Clear();
-            _selectedSearchedCommit = null;
-
-            _requestingWorktreeFiles = false;
-            _worktreeFiles = null;
-            _matchedFilesForSearching = null;
         }
 
         public bool CanCreatePopup()
@@ -864,83 +837,6 @@ namespace SourceGit.ViewModels
             Filter = string.Empty;
         }
 
-        public void ClearSearchCommitFilter()
-        {
-            SearchCommitFilter = string.Empty;
-        }
-
-        public void ClearMatchedFilesForSearching()
-        {
-            MatchedFilesForSearching = null;
-        }
-
-        public void StartSearchCommits()
-        {
-            if (_histories == null)
-                return;
-
-            IsSearchLoadingVisible = true;
-            SelectedSearchedCommit = null;
-            MatchedFilesForSearching = null;
-
-            Task.Run(async () =>
-            {
-                var visible = new List<Models.Commit>();
-                var method = (Models.CommitSearchMethod)_searchCommitFilterType;
-
-                if (method == Models.CommitSearchMethod.BySHA)
-                {
-                    var isCommitSHA = await new Commands.IsCommitSHA(FullPath, _searchCommitFilter)
-                        .GetResultAsync()
-                        .ConfigureAwait(false);
-
-                    if (isCommitSHA)
-                    {
-                        var commit = await new Commands.QuerySingleCommit(FullPath, _searchCommitFilter)
-                            .GetResultAsync()
-                            .ConfigureAwait(false);
-
-                        commit.IsMerged = await new Commands.IsAncestor(FullPath, commit.SHA, "HEAD")
-                            .GetResultAsync()
-                            .ConfigureAwait(false);
-
-                        visible.Add(commit);
-                    }
-                }
-                else if (_onlySearchCommitsInCurrentBranch)
-                {
-                    visible = await new Commands.QueryCommits(FullPath, _searchCommitFilter, method, true)
-                        .GetResultAsync()
-                        .ConfigureAwait(false);
-
-                    foreach (var c in visible)
-                        c.IsMerged = true;
-                }
-                else
-                {
-                    visible = await new Commands.QueryCommits(FullPath, _searchCommitFilter, method, false)
-                        .GetResultAsync()
-                        .ConfigureAwait(false);
-
-                    if (visible.Count > 0)
-                    {
-                        var set = await new Commands.QueryCurrentBranchCommitHashes(FullPath, visible[^1].CommitterTime)
-                            .GetResultAsync()
-                            .ConfigureAwait(false);
-
-                        foreach (var c in visible)
-                            c.IsMerged = set.Contains(c.SHA);
-                    }
-                }
-
-                Dispatcher.UIThread.Post(() =>
-                {
-                    SearchedCommits = visible;
-                    IsSearchLoadingVisible = false;
-                });
-            });
-        }
-
         public IDisposable LockWatcher()
         {
             return _watcher?.Lock();
@@ -985,10 +881,10 @@ namespace SourceGit.ViewModels
             {
                 _navigateToCommitDelayed = sha;
             }
-            else if (_histories != null)
+            else
             {
                 SelectedViewIndex = 0;
-                _histories.NavigateTo(sha);
+                _histories?.NavigateTo(sha);
             }
         }
 
@@ -1003,10 +899,10 @@ namespace SourceGit.ViewModels
             return (_histories?.DetailContext as CommitDetail)?.Commit;
         }
 
-        public void ClearHistoriesFilter()
+        public void ClearHistoryFilters()
         {
-            _settings.HistoriesFilters.Clear();
-            HistoriesFilterMode = Models.FilterMode.None;
+            _historyFilterCollection.Filters.Clear();
+            HistoryFilterMode = Models.FilterMode.None;
 
             ResetBranchTreeFilterMode(LocalBranchTrees);
             ResetBranchTreeFilterMode(RemoteBranchTrees);
@@ -1014,12 +910,12 @@ namespace SourceGit.ViewModels
             RefreshCommits();
         }
 
-        public void RemoveHistoriesFilter(Models.Filter filter)
+        public void RemoveHistoryFilter(Models.HistoryFilter filter)
         {
-            if (_settings.HistoriesFilters.Remove(filter))
+            if (_historyFilterCollection.Filters.Remove(filter))
             {
-                HistoriesFilterMode = _settings.HistoriesFilters.Count > 0 ? _settings.HistoriesFilters[0].Mode : Models.FilterMode.None;
-                RefreshHistoriesFilters(true);
+                HistoryFilterMode = _historyFilterCollection.Mode;
+                RefreshHistoryFilters(true);
             }
         }
 
@@ -1041,9 +937,9 @@ namespace SourceGit.ViewModels
 
         public void SetTagFilterMode(Models.Tag tag, Models.FilterMode mode)
         {
-            var changed = _settings.UpdateHistoriesFilter(tag.Name, Models.FilterType.Tag, mode);
+            var changed = _historyFilterCollection.Update(tag.Name, Models.FilterType.Tag, mode);
             if (changed)
-                RefreshHistoriesFilters(true);
+                RefreshHistoryFilters(true);
         }
 
         public void SetBranchFilterMode(Models.Branch branch, Models.FilterMode mode, bool clearExists, bool refresh)
@@ -1060,28 +956,28 @@ namespace SourceGit.ViewModels
 
             if (clearExists)
             {
-                _settings.HistoriesFilters.Clear();
-                HistoriesFilterMode = Models.FilterMode.None;
+                _historyFilterCollection.Filters.Clear();
+                HistoryFilterMode = Models.FilterMode.None;
             }
 
             if (node.Backend is Models.Branch branch)
             {
                 var type = isLocal ? Models.FilterType.LocalBranch : Models.FilterType.RemoteBranch;
-                var changed = _settings.UpdateHistoriesFilter(node.Path, type, mode);
+                var changed = _historyFilterCollection.Update(node.Path, type, mode);
                 if (!changed)
                     return;
 
                 if (isLocal && !string.IsNullOrEmpty(branch.Upstream) && !branch.IsUpstreamGone)
-                    _settings.UpdateHistoriesFilter(branch.Upstream, Models.FilterType.RemoteBranch, mode);
+                    _historyFilterCollection.Update(branch.Upstream, Models.FilterType.RemoteBranch, mode);
             }
             else
             {
                 var type = isLocal ? Models.FilterType.LocalBranchFolder : Models.FilterType.RemoteBranchFolder;
-                var changed = _settings.UpdateHistoriesFilter(node.Path, type, mode);
+                var changed = _historyFilterCollection.Update(node.Path, type, mode);
                 if (!changed)
                     return;
 
-                _settings.RemoveChildrenBranchFilters(node.Path);
+                _historyFilterCollection.RemoveBranchFiltersByPrefix(node.Path);
             }
 
             var parentType = isLocal ? Models.FilterType.LocalBranchFolder : Models.FilterType.RemoteBranchFolder;
@@ -1097,11 +993,11 @@ namespace SourceGit.ViewModels
                 if (parent == null)
                     break;
 
-                _settings.UpdateHistoriesFilter(parent.Path, parentType, Models.FilterMode.None);
+                _historyFilterCollection.Update(parent.Path, parentType, Models.FilterMode.None);
                 cur = parent;
             } while (true);
 
-            RefreshHistoriesFilters(refresh);
+            RefreshHistoryFilters(refresh);
         }
 
         public async Task StashAllAsync(bool autoStart)
@@ -1293,7 +1189,7 @@ namespace SourceGit.ViewModels
                 if (_settings.HistoryShowFlags.HasFlag(Models.HistoryShowFlags.SimplifyByDecoration))
                     builder.Append("--simplify-by-decoration ");
 
-                var filters = _settings.BuildHistoriesFilter();
+                var filters = _historyFilterCollection.Build();
                 if (string.IsNullOrEmpty(filters))
                     builder.Append("--branches --remotes --tags HEAD");
                 else
@@ -1519,7 +1415,7 @@ namespace SourceGit.ViewModels
         {
             if (_histories != null)
             {
-                SelectedSearchedCommit = null;
+                _searchCommitContext.Selected = null;
 
                 var target = await new Commands.QuerySingleCommit(FullPath, branch.Head).GetResultAsync();
                 _histories.AutoSelectedCommit = null;
@@ -1759,9 +1655,9 @@ namespace SourceGit.ViewModels
                 builder.Run(visibles, remotes, true);
             }
 
-            var historiesFilters = _settings.CollectHistoriesFilters();
-            UpdateBranchTreeFilterMode(builder.Locals, historiesFilters);
-            UpdateBranchTreeFilterMode(builder.Remotes, historiesFilters);
+            var filterMap = _historyFilterCollection.ToMap();
+            UpdateBranchTreeFilterMode(builder.Locals, filterMap);
+            UpdateBranchTreeFilterMode(builder.Remotes, filterMap);
             return builder;
         }
 
@@ -1791,21 +1687,21 @@ namespace SourceGit.ViewModels
                 }
             }
 
-            var historiesFilters = _settings.CollectHistoriesFilters();
-            UpdateTagFilterMode(historiesFilters);
+            var filterMap = _historyFilterCollection.ToMap();
+            UpdateTagFilterMode(filterMap);
 
             if (Preferences.Instance.ShowTagsAsTree)
             {
                 var tree = TagCollectionAsTree.Build(visible, _visibleTags as TagCollectionAsTree);
                 foreach (var node in tree.Tree)
-                    node.UpdateFilterMode(historiesFilters);
+                    node.UpdateFilterMode(filterMap);
                 return tree;
             }
             else
             {
                 var list = new TagCollectionAsList(visible);
                 foreach (var item in list.TagItems)
-                    item.FilterMode = historiesFilters.GetValueOrDefault(item.Tag.Name, Models.FilterMode.None);
+                    item.FilterMode = filterMap.GetValueOrDefault(item.Tag.Name, Models.FilterMode.None);
                 return list;
             }
         }
@@ -1832,45 +1728,41 @@ namespace SourceGit.ViewModels
                 return new SubmoduleCollectionAsList() { Submodules = visible };
         }
 
-        private void RefreshHistoriesFilters(bool refresh)
+        private void RefreshHistoryFilters(bool refresh)
         {
-            if (_settings.HistoriesFilters.Count > 0)
-                HistoriesFilterMode = _settings.HistoriesFilters[0].Mode;
-            else
-                HistoriesFilterMode = Models.FilterMode.None;
-
+            HistoryFilterMode = _historyFilterCollection.Mode;
             if (!refresh)
                 return;
 
-            var filters = _settings.CollectHistoriesFilters();
-            UpdateBranchTreeFilterMode(LocalBranchTrees, filters);
-            UpdateBranchTreeFilterMode(RemoteBranchTrees, filters);
-            UpdateTagFilterMode(filters);
+            var map = _historyFilterCollection.ToMap();
+            UpdateBranchTreeFilterMode(LocalBranchTrees, map);
+            UpdateBranchTreeFilterMode(RemoteBranchTrees, map);
+            UpdateTagFilterMode(map);
             RefreshCommits();
         }
 
-        private void UpdateBranchTreeFilterMode(List<BranchTreeNode> nodes, Dictionary<string, Models.FilterMode> filters)
+        private void UpdateBranchTreeFilterMode(List<BranchTreeNode> nodes, Dictionary<string, Models.FilterMode> map)
         {
             foreach (var node in nodes)
             {
-                node.FilterMode = filters.GetValueOrDefault(node.Path, Models.FilterMode.None);
+                node.FilterMode = map.GetValueOrDefault(node.Path, Models.FilterMode.None);
 
                 if (!node.IsBranch)
-                    UpdateBranchTreeFilterMode(node.Children, filters);
+                    UpdateBranchTreeFilterMode(node.Children, map);
             }
         }
 
-        private void UpdateTagFilterMode(Dictionary<string, Models.FilterMode> filters)
+        private void UpdateTagFilterMode(Dictionary<string, Models.FilterMode> map)
         {
             if (VisibleTags is TagCollectionAsTree tree)
             {
                 foreach (var node in tree.Tree)
-                    node.UpdateFilterMode(filters);
+                    node.UpdateFilterMode(map);
             }
             else if (VisibleTags is TagCollectionAsList list)
             {
                 foreach (var item in list.TagItems)
-                    item.FilterMode = filters.GetValueOrDefault(item.Tag.Name, Models.FilterMode.None);
+                    item.FilterMode = map.GetValueOrDefault(item.Tag.Name, Models.FilterMode.None);
             }
         }
 
@@ -1920,65 +1812,6 @@ namespace SourceGit.ViewModels
             return null;
         }
 
-        private bool IsSearchingCommitsByFilePath()
-        {
-            return _isSearching && _searchCommitFilterType == (int)Models.CommitSearchMethod.ByPath;
-        }
-
-        private void CalcWorktreeFilesForSearching()
-        {
-            if (!IsSearchingCommitsByFilePath())
-            {
-                _requestingWorktreeFiles = false;
-                _worktreeFiles = null;
-                MatchedFilesForSearching = null;
-                GC.Collect();
-                return;
-            }
-
-            if (_requestingWorktreeFiles)
-                return;
-
-            _requestingWorktreeFiles = true;
-
-            Task.Run(async () =>
-            {
-                _worktreeFiles = await new Commands.QueryRevisionFileNames(FullPath, "HEAD")
-                    .GetResultAsync()
-                    .ConfigureAwait(false);
-
-                Dispatcher.UIThread.Post(() =>
-                {
-                    if (IsSearchingCommitsByFilePath() && _requestingWorktreeFiles)
-                        CalcMatchedFilesForSearching();
-
-                    _requestingWorktreeFiles = false;
-                });
-            });
-        }
-
-        private void CalcMatchedFilesForSearching()
-        {
-            if (_worktreeFiles == null || _worktreeFiles.Count == 0 || _searchCommitFilter.Length < 3)
-            {
-                MatchedFilesForSearching = null;
-                return;
-            }
-
-            var matched = new List<string>();
-            foreach (var file in _worktreeFiles)
-            {
-                if (file.Contains(_searchCommitFilter, StringComparison.OrdinalIgnoreCase) && file.Length != _searchCommitFilter.Length)
-                {
-                    matched.Add(file);
-                    if (matched.Count > 100)
-                        break;
-                }
-            }
-
-            MatchedFilesForSearching = matched;
-        }
-
         private void FetchInBackground(object sender)
         {
             Dispatcher.UIThread.Invoke(async Task () =>
@@ -2029,7 +1862,8 @@ namespace SourceGit.ViewModels
         private readonly bool _isWorktree = false;
         private readonly string _gitCommonDir = null;
         private Models.RepositorySettings _settings = null;
-        private Models.FilterMode _historiesFilterMode = Models.FilterMode.None;
+        private Models.HistoryFilterCollection _historyFilterCollection = null;
+        private Models.FilterMode _historyFilterMode = Models.FilterMode.None;
         private bool _hasAllowedSignersFile = false;
 
         private Models.Watcher _watcher = null;
@@ -2043,16 +1877,8 @@ namespace SourceGit.ViewModels
         private int _localChangesCount = 0;
         private int _stashesCount = 0;
 
-        private bool _isSearching = false;
-        private bool _isSearchLoadingVisible = false;
-        private int _searchCommitFilterType = (int)Models.CommitSearchMethod.ByMessage;
-        private bool _onlySearchCommitsInCurrentBranch = false;
-        private string _searchCommitFilter = string.Empty;
-        private List<Models.Commit> _searchedCommits = [];
-        private Models.Commit _selectedSearchedCommit = null;
-        private bool _requestingWorktreeFiles = false;
-        private List<string> _worktreeFiles = null;
-        private List<string> _matchedFilesForSearching = null;
+        private bool _isSearchingCommits = false;
+        private SearchCommitContext _searchCommitContext = null;
 
         private string _filter = string.Empty;
         private List<Models.Remote> _remotes = [];
