@@ -438,11 +438,29 @@ namespace SourceGit.Views
                 e.Handled = true;
             };
 
+            var copyMessage = new MenuItem();
+            copyMessage.Header = App.Text("CommitCM.CopyCommitMessage");
+            copyMessage.Icon = App.CreateMenuIcon("Icons.Info");
+            copyMessage.Click += async (_, e) =>
+            {
+                var vm = DataContext as ViewModels.Histories;
+                var messages = new List<string>();
+                foreach (var c in selected)
+                {
+                    var message = await vm.GetCommitFullMessageAsync(c);
+                    messages.Add(message);
+                }
+
+                await App.CopyTextAsync(string.Join("\n-----\n", messages));
+                e.Handled = true;
+            };
+
             var copy = new MenuItem();
             copy.Header = App.Text("Copy");
             copy.Icon = App.CreateMenuIcon("Icons.Copy");
             copy.Items.Add(copyShas);
             copy.Items.Add(copyInfos);
+            copy.Items.Add(copyMessage);
             menu.Items.Add(copy);
             return menu;
         }
@@ -598,21 +616,31 @@ namespace SourceGit.Views
                     };
                     menu.Items.Add(cherryPick);
                 }
-                else
+
+                var revert = new MenuItem();
+                revert.Header = App.Text("CommitCM.Revert");
+                revert.Icon = App.CreateMenuIcon("Icons.Undo");
+                revert.Click += (_, e) =>
                 {
-                    var revert = new MenuItem();
-                    revert.Header = App.Text("CommitCM.Revert");
-                    revert.Icon = App.CreateMenuIcon("Icons.Undo");
-                    revert.Click += (_, e) =>
+                    if (repo.CanCreatePopup())
+                        repo.ShowPopup(new ViewModels.Revert(repo, commit));
+                    e.Handled = true;
+                };
+                menu.Items.Add(revert);
+
+                if (isHead)
+                {
+                    var dropHead = new MenuItem();
+                    dropHead.Header = App.Text("CommitCM.Drop");
+                    dropHead.Icon = App.CreateMenuIcon("Icons.Clear");
+                    dropHead.Click += async (_, e) =>
                     {
-                        if (repo.CanCreatePopup())
-                            repo.ShowPopup(new ViewModels.Revert(repo, commit));
+                        await vm.DropHeadAsync(commit);
                         e.Handled = true;
                     };
-                    menu.Items.Add(revert);
+                    menu.Items.Add(dropHead);
                 }
-
-                if (!isHead)
+                else
                 {
                     var checkoutCommit = new MenuItem();
                     checkoutCommit.Header = App.Text("CommitCM.Checkout");
@@ -861,7 +889,8 @@ namespace SourceGit.Views
             copyMessage.Icon = App.CreateMenuIcon("Icons.Info");
             copyMessage.Click += async (_, e) =>
             {
-                await vm.CopyCommitFullMessageAsync(commit);
+                var message = await vm.GetCommitFullMessageAsync(commit);
+                await App.CopyTextAsync(message);
                 e.Handled = true;
             };
 
