@@ -8,6 +8,7 @@ using Avalonia.Collections;
 using Avalonia.Controls;
 using Avalonia.Controls.Primitives;
 using Avalonia.Input;
+using Avalonia.Interactivity;
 using Avalonia.Layout;
 using Avalonia.Media;
 using Avalonia.Platform.Storage;
@@ -301,36 +302,21 @@ namespace SourceGit.Views
                 _tree.Clear();
                 _searchResult.Clear();
 
-                var vm = DataContext as ViewModels.CommitDetail;
-                if (vm?.Commit == null)
-                {
+                if (DataContext is ViewModels.CommitDetail { ActiveTabIndex: 2 } vm)
+                    await ReloadTreeData(vm);
+                else
                     Rows.Clear();
-                    GC.Collect();
-                    return;
-                }
-
-                var objects = await vm.GetRevisionFilesUnderFolderAsync(null);
-                if (objects == null || objects.Count == 0)
-                {
-                    Rows.Clear();
-                    GC.Collect();
-                    return;
-                }
-
-                foreach (var obj in objects)
-                    _tree.Add(new ViewModels.RevisionFileTreeNode { Backend = obj });
-
-                SortNodes(_tree);
-
-                var topTree = new List<ViewModels.RevisionFileTreeNode>();
-                MakeRows(topTree, _tree, 0);
-
-                Rows.Clear();
-                Rows.AddRange(topTree);
-                GC.Collect();
             }
         }
 
+        protected override async void OnLoaded(RoutedEventArgs e)
+        {
+            base.OnLoaded(e);
+
+            if (DataContext is ViewModels.CommitDetail vm && _tree.Count == 0)
+                await ReloadTreeData(vm);
+        }
+
         private void OnTreeNodeContextRequested(object sender, ContextRequestedEventArgs e)
         {
             if (DataContext is ViewModels.CommitDetail { Repository: { } repo, Commit: { } commit } vm &&
@@ -416,7 +402,42 @@ namespace SourceGit.Views
             });
         }
 
-        public ContextMenu CreateRevisionFileContextMenuByFolder(ViewModels.Repository repo, ViewModels.CommitDetail vm, Models.Commit commit, string path)
+        private async Task ReloadTreeData(ViewModels.CommitDetail vm)
+        {
+            if (_isReloadingTreeData)
+                return;
+
+            _isReloadingTreeData = true;
+
+            if (vm?.Commit == null)
+            {
+                Rows.Clear();
+                _isReloadingTreeData = false;
+                return;
+            }
+
+            var objects = await vm.GetRevisionFilesUnderFolderAsync(null);
+            if (objects == null || objects.Count == 0)
+            {
+                Rows.Clear();
+                _isReloadingTreeData = false;
+                return;
+            }
+
+            foreach (var obj in objects)
+                _tree.Add(new ViewModels.RevisionFileTreeNode { Backend = obj });
+
+            SortNodes(_tree);
+
+            var topTree = new List<ViewModels.RevisionFileTreeNode>();
+            MakeRows(topTree, _tree, 0);
+
+            Rows.Clear();
+            Rows.AddRange(topTree);
+            _isReloadingTreeData = false;
+        }
+
+        private ContextMenu CreateRevisionFileContextMenuByFolder(ViewModels.Repository repo, ViewModels.CommitDetail vm, Models.Commit commit, string path)
         {
             var fullPath = Native.OS.GetAbsPath(repo.FullPath, path);
             var explore = new MenuItem();
@@ -468,21 +489,49 @@ namespace SourceGit.Views
             return menu;
         }
 
-        public ContextMenu CreateRevisionFileContextMenu(ViewModels.Repository repo, ViewModels.CommitDetail vm, Models.Commit commit, Models.Object file)
+        private ContextMenu CreateRevisionFileContextMenu(ViewModels.Repository repo, ViewModels.CommitDetail vm, Models.Commit commit, Models.Object file)
         {
             var fullPath = Native.OS.GetAbsPath(repo.FullPath, file.Path);
             var menu = new ContextMenu();
 
             var openWith = new MenuItem();
-            openWith.Header = App.Text("OpenWith");
+            openWith.Header = App.Text("Open");
             openWith.Icon = App.CreateMenuIcon("Icons.OpenWith");
-            openWith.Tag = OperatingSystem.IsMacOS() ? "⌘+O" : "Ctrl+O";
             openWith.IsEnabled = file.Type == Models.ObjectType.Blob;
-            openWith.Click += async (_, ev) =>
+            if (openWith.IsEnabled)
             {
-                await vm.OpenRevisionFileWithDefaultEditorAsync(file.Path);
-                ev.Handled = true;
-            };
+                var defaultEditor = new MenuItem();
+                defaultEditor.Header = App.Text("Open.SystemDefaultEditor");
+                defaultEditor.Tag = OperatingSystem.IsMacOS() ? "⌘+O" : "Ctrl+O";
+                defaultEditor.Click += async (_, ev) =>
+                {
+                    await vm.OpenRevisionFileAsync(file.Path, null);
+                    ev.Handled = true;
+                };
+
+                openWith.Items.Add(defaultEditor);
+
+                var tools = Native.OS.ExternalTools;
+                if (tools.Count > 0)
+                {
+                    openWith.Items.Add(new MenuItem() { Header = "-" });
+
+                    for (var i = 0; i < tools.Count; i++)
+                    {
+                        var tool = tools[i];
+                        var item = new MenuItem();
+                        item.Header = tool.Name;
+                        item.Icon = new Image { Width = 16, Height = 16, Source = tool.IconImage };
+                        item.Click += async (_, ev) =>
+                        {
+                            await vm.OpenRevisionFileAsync(file.Path, tool);
+                            ev.Handled = true;
+                        };
+
+                        openWith.Items.Add(item);
+                    }
+                }
+            }
 
             var saveAs = new MenuItem();
             saveAs.Header = App.Text("SaveAs");
@@ -646,6 +695,33 @@ namespace SourceGit.Views
                 }
             }
 
+            var actions = repo.GetCustomActions(Models.CustomActionScope.File);
+            if (actions.Count > 0)
+            {
+                var target = new Models.CustomActionTargetFile(file.Path, vm.Commit);
+                var custom = new MenuItem();
+                custom.Header = App.Text("FileCM.CustomAction");
+                custom.Icon = App.CreateMenuIcon("Icons.Action");
+
+                foreach (var action in actions)
+                {
+                    var (dup, label) = action;
+                    var item = new MenuItem();
+                    item.Icon = App.CreateMenuIcon("Icons.Action");
+                    item.Header = label;
+                    item.Click += async (_, e) =>
+                    {
+                        await repo.ExecCustomActionAsync(dup, target);
+                        e.Handled = true;
+                    };
+
+                    custom.Items.Add(item);
+                }
+
+                menu.Items.Add(custom);
+                menu.Items.Add(new MenuItem() { Header = "-" });
+            }
+
             var copyPath = new MenuItem();
             copyPath.Header = App.Text("CopyPath");
             copyPath.Icon = App.CreateMenuIcon("Icons.Copy");
@@ -674,5 +750,6 @@ namespace SourceGit.Views
         private List<ViewModels.RevisionFileTreeNode> _tree = [];
         private bool _disableSelectionChangingEvent = false;
         private List<ViewModels.RevisionFileTreeNode> _searchResult = [];
+        private bool _isReloadingTreeData = false;
     }
 }
